/**
*
* DIALOGIC CONFIDENTIAL 
* Copyright © 2012 Dialogic Inc. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to 
* the source code (collectively "Material") are owned by Dialogic Inc. and/or its subsidiaries, affiliates, suppliers
* or licensors (collectively "Dialogic"), with whom title to the Material remains. The Material contains trade secrets and proprietary
* and confidential information of Dialogic. The
* Material is protected by worldwide patent, copyright, trade secret and/or other applicable intellectual property laws and treaty
* provisions. No part of the Material may be used, copied, reproduced, 
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret and/or other intellectual 
* property right is granted to or conferred upon you by disclosure or delivery
* of the Material, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

/*!
 * \file   document.cxx
 * \brief  Documents are containers sets of elements.
 * \author John Tarlton
 */

#include <iostream>

#include "element.h"
#include "document.h"


/*
 * ctor
 */
Document::Document( const std::string& id ) : id_ (id)
{
}


/*
 * dtor, delete all elements.
 */
Document::~Document()
{
	std::vector<Element*>::iterator i = elements_.begin();
	for ( ; i != elements_.end(); ++i )
	{
		delete *i;
	}
}


/*
 * Add an element to the map using its id.
 */
bool Document::addElement( Element* el )
{
	if ( el && !findElement(el->getId()) )
	{
		elements_.push_back(el);
		return true;
	}
	return false;
}


/*
 * Lookup an element by its id.
 */
Element* Document::findElement( const std::string& id ) const
{
	std::vector<Element*>::const_iterator i = elements_.begin();
	for ( ; i != elements_.end(); ++i )
	{
		if ( (*i)->getId() == id )
		{
			return *i;
		}
	}
	return 0;
}


/*
 * Lookup the element and compare its type against 'type'.
 */
bool Document::isElementType( const std::string& id,
                              element_type_t type ) const
{
	Element* el = findElement(id);
	if ( el )
	{
		return (el->getType() == type);
	}
	return false;
}


/*
 * Clear the content of all elements.
 */
void Document::clearContent()
{
	std::vector<Element*>::iterator i = elements_.begin();
	for ( ; i != elements_.end(); ++i )
	{
		(*i)->clear();
	}
}


/*
 * Lookup the element and pass it the content.
 */
bool Document::addContent( const std::string& id, const std::string& content )
{
	Element* el = findElement(id);
	if ( el )
	{
		el->append(content);
		return true;
	}
	return false;
}


/*
 * Set the document's style.
 */
bool Document::setStyle( const Style& style )
{
	style_ = style;
	return true;
}


/*
 * Set the style of an element.
 */
bool Document::setStyle( const std::string& id, const Style& style )
{
	Element* el = findElement(id);
	if ( el )
	{
		el->setStyle(style);
		return true;
	}
	return false;
}


/*
 * Create the base image and call each element to draw itself.
 */
bool Document::draw( const std::string& filename, unsigned int width, unsigned int height )
{
	try
	{
		Magick::Image image(Magick::Geometry(style_.getWidth(),
		                                     style_.getHeight()),
		                                     style_.getBackgroundColor());

		if ( !style_.getBackgroundImage().empty() )
		{
			image.read(style_.getBackgroundImage());
			image.crop(Magick::Geometry(style_.getWidth(), style_.getHeight()));
		}

		std::list<Magick::Drawable> draw_list;

		draw_list.push_back(Magick::DrawableStrokeAntialias(true));

		std::vector<Element*>::const_iterator i = elements_.begin();
		for ( ; i != elements_.end(); ++i )
		{
			(*i)->draw(draw_list);
		}

		image.draw(draw_list);
		image.quality(100);

		if ( width && height )
		{
			image.scale(Magick::Geometry(width, height));
		}
		image.write(filename);
	}
	catch( std::exception& error )
	{
		std::cerr << "Document::draw() Caught exception: " << error.what() << std::endl;
		return false;
	}
	return true;
}


/* vim:ts=4:set nu:
 * EOF
 */
