/**
*
* DIALOGIC CONFIDENTIAL 
* Copyright © 2012 Dialogic Inc. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to 
* the source code (collectively "Material") are owned by Dialogic Inc. and/or its subsidiaries, affiliates, suppliers
* or licensors (collectively "Dialogic"), with whom title to the Material remains. The Material contains trade secrets and proprietary
* and confidential information of Dialogic. The
* Material is protected by worldwide patent, copyright, trade secret and/or other applicable intellectual property laws and treaty
* provisions. No part of the Material may be used, copied, reproduced, 
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret and/or other intellectual 
* property right is granted to or conferred upon you by disclosure or delivery
* of the Material, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

/*!
 * \file   document.h
 * \brief  Documents are containers sets of elements.
 * \author John Tarlton
 */

#ifndef _DOCUMENT_H
#define _DOCUMENT_H

#include <string>
#include <map>

#include "style.h"

class Element;

/*!
 * \class Document
 */
class Document
{
public:

	/*!
	 * ctor
	 * \param id - unique identifer for this object.
	 */
	Document( const std::string& id );

	/*!
	 * dtor
	 */
	virtual ~Document();

	/*!
	 * Take ownership of an elemnt and add it to the document.
	 * \param el - element object.
	 */
	bool addElement( Element* el );

	/*!
	 * Test an element's type.
	 * \param id - unique identifer for the element.
	 * \param type - value to test.
	 */
	bool isElementType( const std::string& id, element_type_t type ) const;

	/*!
	 * Set the document's style.
	 * \param style - styling information.
	 */
	bool setStyle( const Style& style );

	/*!
	 * Set the style of the named element.
	 * \param id - unique identifer for the element.
	 * \param style - styling information.
	 */
	bool setStyle( const std::string& id, const Style& style );

	/*!
	 * Get the document's style.
	 */
	const Style& getStyle() const { return style_; }

	/*!
	 * Set the width of the image canvas.
	 * \param width - width in pixels.
	 */
	void setCanvasWidth( unsigned int width ) { canvas_width_ = width; }

	/*!
	 * Get the width of the image canvas.
	 * \param width - width in pixels.
	 */
	unsigned int getCanvasWidth() const { return canvas_width_; }

	/*!
	 * Set the width of the image canvas.
	 * \param height - height in pixels.
	 */
	void setCanvasHeight( unsigned int height ) { canvas_height_ = height; }

	/*!
	 * Get the height of the image canvas.
	 */
	unsigned int getCanvasHeight() const { return canvas_height_; }

	/*!
	 * Clear all content.
	 */
	void clearContent();

	/*!
	 * Add content to an element.
	 * \param id - unique identifer for the element.
	 * \param content - content for the element.
	 */
	bool addContent( const std::string& id, const std::string& content );

	/*!
	 * Render the document as an image and write out to a file.
	 * \param filename - output file.
	 * \param width - if non-zero this overrides the width of the image.
	 * \param height - if non-zero this overrides the height of the image.
	 */
	bool draw( const std::string& filename, unsigned int width, unsigned int height );

protected:

	/*!
	 * Lookup and element's object by its id.
	 */
	Element* findElement( const std::string& id ) const;

protected:

	std::string id_;   /*!< uniquely identifies this instance. */

	Style style_;      /*!< base style, inherited by elements. */

	unsigned int canvas_width_;
	unsigned int canvas_height_;

	/*! 
	 * Elements are stored in the order that they are added, this gives them a
	 * bottom to top z-order when drawn.
	 */
	std::vector<Element*> elements_;
};


#endif // _DOCUMENT_H

/* vim:ts=4:set nu:
 * EOF
 */
