/**
*
* DIALOGIC CONFIDENTIAL 
* Copyright © 2012 Dialogic Inc. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to 
* the source code (collectively "Material") are owned by Dialogic Inc. and/or its subsidiaries, affiliates, suppliers
* or licensors (collectively "Dialogic"), with whom title to the Material remains. The Material contains trade secrets and proprietary
* and confidential information of Dialogic. The
* Material is protected by worldwide patent, copyright, trade secret and/or other applicable intellectual property laws and treaty
* provisions. No part of the Material may be used, copied, reproduced, 
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret and/or other intellectual 
* property right is granted to or conferred upon you by disclosure or delivery
* of the Material, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

/*!
 * \file   element.cxx
 * \brief  Elements are a the basic building blocks used to compose ImageMaker
 *         documents. Sub classes are used to store different types of content
 *         such as text strings or image files and to render them in specific ways.
 * \author John Tarlton
 */
#include <iostream>
#include "element.h"
#include "style.h"


/*
 * Draw the element's background.
 */
void Element::drawBackground( std::list<Magick::Drawable>& draw_list )
{
	draw_list.push_back(Magick::DrawableStrokeWidth(style_.getBorderWidth()));
	draw_list.push_back(Magick::DrawableStrokeColor(style_.getBorderColor()));

	draw_list.push_back(Magick::DrawableFillColor(style_.getBackgroundColor()));

	unsigned int left = style_.getLeft();
	unsigned int right = style_.getWidth() + left;
	unsigned int top =  style_.getTop();
	unsigned int bottom = top + style_.getHeight();

	if ( style_.getBorderRadius() > 0 )
	{
		draw_list.push_back(Magick::DrawableRoundRectangle(left,
		                                                   top,
		                                                   right,
		                                                   bottom,
		                                                   style_.getBorderRadius(),
		                                                   style_.getBorderRadius()));
	}
	else
	{
		draw_list.push_back(Magick::DrawableRectangle(left,
		                                              top,
		                                              right,
		                                              bottom));
	}

	/* The image will appear inside the background rectangle. For images that
	 * support transparency, such as png, the background fill colour will show
	 * through.
	 */
	if ( !style_.getBackgroundImage().empty() )
	{
		draw_list.push_back(Magick::DrawableCompositeImage(0,
		                                                   style_.getTop(),
		                                                   style_.getWidth() - style_.getBorderWidth(),
		                                                   style_.getHeight(),
		                                                   style_.getBackgroundImage(),
		                                                   Magick::PlusCompositeOp));
	}
}

///////////////////////////////////////////////////////////////////////////////

/*
 * Add content to the current buffer.
 */
void TextElement::append( const std::string& content )
{
	content_.append(content);
}


/*
 * Remove all content.
 */
void TextElement::clear()
{
	content_.clear();
}


/*
 * Add the operations required to draw this element to the drawing list.
 */
bool TextElement::draw( std::list<Magick::Drawable>& draw_list )
{
	drawBackground(draw_list);

	if ( !content_.empty() )
	{
		draw_list.push_back(Magick::DrawablePointSize(style_.getFontSize()));
		draw_list.push_back(Magick::DrawableStrokeWidth(0));
		draw_list.push_back(Magick::DrawableStrokeColor(style_.getColor()));
		draw_list.push_back(Magick::DrawableFillColor(style_.getColor()));
		draw_list.push_back(Magick::DrawableGravity(style_.getTextAlign()));

		if ( style_.getTextAlign() == Magick::NorthGravity )
		{
			/* centre (of image) */
			draw_list.push_back(Magick::DrawableText(0,
			                                         style_.getTop() + style_.getPadding(),
			                                         content_));
		}
		else if ( style_.getTextAlign() == Magick::NorthWestGravity )
		{
			/* left */
			draw_list.push_back(Magick::DrawableText(style_.getLeft() + style_.getPadding(),
			                                         style_.getTop() + style_.getPadding(),
			                                         content_));
		}
		else
		{
			/* right */
			draw_list.push_back(Magick::DrawableText(style_.getPadding(),
			                                         style_.getTop() + style_.getPadding(),
			                                         content_));
		}
	}
	return true;
}

///////////////////////////////////////////////////////////////////////////////

/*
 * Append content as a new list item.
 */
void ListElement::append( const std::string& content )
{
	contents_.push_back(content);
}


/*
 * Remove all content.
 */
void ListElement::clear()
{
	contents_.clear();
}


/*
 * Add the operations required to draw this element to the drawing list.
 */
bool ListElement::draw( std::list<Magick::Drawable>& draw_list )
{
	drawBackground(draw_list);

	if ( !contents_.empty() )
	{
		draw_list.push_back(Magick::DrawablePointSize(style_.getFontSize()));
 		draw_list.push_back(Magick::DrawableStrokeWidth(0));
		draw_list.push_back(Magick::DrawableStrokeColor(style_.getColor()));
		draw_list.push_back(Magick::DrawableFillColor(style_.getColor()));
		draw_list.push_back(Magick::DrawableGravity(style_.getTextAlign()));

		int row_offset = style_.getTop() + style_.getPadding();
		int row_pitch = style_.getFontSize() + style_.getFontSize() / 2;
		int row_count = style_.getHeight() / row_pitch;

		int col_count = (contents_.size() - 1) / row_count + 1;
		int col_offset = col_count == 1 ? style_.getWidth() / 3 : 30;
		int col_pitch = style_.getWidth() / col_count;

		std::vector<std::string>::const_iterator i;
		int index = 0;
		for ( i = contents_.begin(); i != contents_.end(); ++i )
		{
			draw_list.push_back(Magick::DrawablePointSize(style_.getFontSize()));
			draw_list.push_back(Magick::DrawableStrokeColor(style_.getColor()));
			draw_list.push_back(Magick::DrawableFillColor(style_.getColor()));

			int x = index / row_count * col_pitch + col_offset;
			int y = index % row_count * row_pitch + row_offset;
			draw_list.push_back(Magick::DrawableText(x, y, *i));
			index++;
		}
	}
	return true;
}

///////////////////////////////////////////////////////////////////////////////

/*
 * Overwrite the image filename.
 */
void ImageElement::append( const std::string& content )
{
	content_ = content;
}


/*
 * Clear the image filename.
 */
void ImageElement::clear()
{
	content_.clear();
}


/*
 * Add the operations required to draw this element to the drawing list.
 */
bool ImageElement::draw( std::list<Magick::Drawable>& draw_list )
{
	drawBackground(draw_list);

	if ( !content_.empty() )
	{
		draw_list.push_back(Magick::DrawableGravity(Magick::ForgetGravity));

		draw_list.push_back(Magick::DrawableCompositeImage(style_.getLeft(),
		                                                   style_.getTop(),
		                                                   style_.getWidth(),
		                                                   style_.getHeight(),
		                                                   content_,
		                                                   Magick::PlusCompositeOp));
	}
	return true;
}


/* vim:ts=4:set nu:
 * EOF
 */
