/**
*
* DIALOGIC CONFIDENTIAL 
* Copyright © 2012 Dialogic Inc. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to 
* the source code (collectively "Material") are owned by Dialogic Inc. and/or its subsidiaries, affiliates, suppliers
* or licensors (collectively "Dialogic"), with whom title to the Material remains. The Material contains trade secrets and proprietary
* and confidential information of Dialogic. The
* Material is protected by worldwide patent, copyright, trade secret and/or other applicable intellectual property laws and treaty
* provisions. No part of the Material may be used, copied, reproduced, 
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret and/or other intellectual 
* property right is granted to or conferred upon you by disclosure or delivery
* of the Material, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

/*!
 * \file   element.h
 * \brief  Elements are a the basic building blocks used to compose ImageMaker
 *         documents. Sub classes are used to store different types of content
 *         such as text strings or image files and to render them in specific ways.
 * \author John Tarlton
 */

#ifndef _ELEMENT_H
#define _ELEMENT_H

#include <string>
#include <vector>

#include <Magick++.h>

#include "style.h"


/*!
 * \enum ElementTypes - element type identifiers.
 */
enum ElementTypes
{
	ETYPE_TEXT,
	ETYPE_LIST,
	ETYPE_IMAGE
};

typedef ElementTypes element_type_t;


/*!
 * \class Element
 * Base class for all document elements.
 */
class Element
{
public:

	/*!
	 * ctor, set the type and unique identifer.
	 */
	Element( element_type_t type, const std::string& id )
	 : type_ (type), id_ (id) {;}

	/*!
	 *	dtor
	 */
	virtual ~Element() {;}

	/*!
	 * Get the type of this object.
	 */
	const element_type_t& getType() const { return type_; }

	/*!
	 * Get the unique id of this object.
	 */
	const std::string& getId() const { return id_; }

	/*!
	 * Set the style information.
	 */
	void setStyle( const Style& style ) { style_ = style; }

	/*!
	 * Get the style information of this object.
	 */
	const Style& getStyle() const { return style_; }

	/*!
	 * Implemented by sub-classes to add content.
	 */
	virtual void append( const std::string& value ) = 0;

	/*!
	 * Implemented by sub-classes to remove all content.
	 */
	virtual void clear() = 0;

	/*!
	 * Implemented by sub-classes to draw themselves.
	 */
	virtual bool draw( std::list<Magick::Drawable>& draw_list ) = 0;

protected:

	void drawBackground( std::list<Magick::Drawable>& draw_list );

private:

	/*
	 * Element objects must have a unique id.
	 */
	Element();
	Element( Element const& );
	Element& operator = ( Element const& );

protected:

	element_type_t type_;  /*!< explicit type identifier */
	std::string    id_;    /*!< uniquely identifies this instance. */

	Style style_;          /*!< styling information */
};


/*!
 * \class TextElement
 * A text element consists of a single string.
 */
class TextElement : public Element
{
public:

	TextElement( const std::string& id )
	  : Element(ETYPE_TEXT, id) {;}

	virtual ~TextElement() {;}

	virtual void append( const std::string& content );

	virtual void clear();

	virtual bool draw( std::list<Magick::Drawable>& draw_list );

private:

	TextElement();
	TextElement( TextElement const& );
	TextElement& operator = ( TextElement const& );

private:

	std::string content_;  /*!< literal text to display */
};


/*!
 * \class ListElement
 * A list element consists of a number of text strings that are rendered as
 * multiple rows in one or more columns.
 * e.g.
 *     1   4
 *     2   5
 *     3
 */
class ListElement : public Element
{
public:

	ListElement( const std::string& id )
	  : Element(ETYPE_LIST, id) {;}

	virtual ~ListElement() {;}

	virtual void append( const std::string& content );

	virtual void clear();

	virtual bool draw( std::list<Magick::Drawable>& draw_list );

private:

	ListElement();
	ListElement( TextElement const& );
	ListElement& operator = ( ListElement const& );

private:

	std::vector<std::string> contents_;   /*!< literal text for each item */
};


/*!
 * \class ImageElement
 * An image element consists of a single image file.
 * The file may be of any format that is supported by the ImageMagick library.
 */
class ImageElement : public Element
{
public:

	ImageElement( const std::string& id )
	  : Element(ETYPE_IMAGE, id) {;}

	virtual ~ImageElement() {;}

	virtual void append( const std::string& content );

	virtual void clear();

	virtual bool draw( std::list<Magick::Drawable>& draw_list );

private:

	ImageElement();
	ImageElement( ImageElement const& );
	ImageElement& operator = ( ImageElement const& );

private:

	std::string content_;   /*!< image file name */
};

#endif  // _ELEMENT_H

/* vim:ts=4:set nu:
 * EOF
 */
