/**
*
* DIALOGIC CONFIDENTIAL 
* Copyright © 2012 Dialogic Inc. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to 
* the source code (collectively "Material") are owned by Dialogic Inc. and/or its subsidiaries, affiliates, suppliers
* or licensors (collectively "Dialogic"), with whom title to the Material remains. The Material contains trade secrets and proprietary
* and confidential information of Dialogic. The
* Material is protected by worldwide patent, copyright, trade secret and/or other applicable intellectual property laws and treaty
* provisions. No part of the Material may be used, copied, reproduced, 
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret and/or other intellectual 
* property right is granted to or conferred upon you by disclosure or delivery
* of the Material, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

/*!
 * \file   imagemaker.cxx
 * \brief  ImageMaker is a tool for creating image files by combining document
 *         templates and style information that are read in from configuration
 *         files with run-time generated content data.
 * \author John Tarlton
 */

#include <iostream>

#include "imagemaker.h"


/*
 * ctor
 */
ImageMaker::ImageMaker()
{
	;
}


/*
 * dtor, delete all docs.
 */
ImageMaker::~ImageMaker()
{
	document_map_t::iterator i = docs_.begin();
	for ( ; i != docs_.end(); ++i )
	{
		delete (*i).second;
	}
}


/*
 * Load configuration, possibly from multiple files. This will build the
 * document objects. Each conf file section defines one document.
 * Example, a document called 'menu':
 *
 *  [menu]
 *
 *  ; Name of the file containing the style information for this doc.
 *  ;
 *  style = menu.style
 *
 *  ; Elements, defined as: type = id
 *  ; Valid types are: text and list.
 *  ;
 *  text = header
 *  list = items
 *  text = footer
 */
bool ImageMaker::init( const std::string& dir_name, const std::string& image_dir )
{
	image_dir_ = image_dir.empty() ? "." : image_dir;

	IniFile ini;
	ini.loadMultiple(dir_name, "conf");

	std::vector<std::string> names;
	ini.getSectionNames(names);        /* section name => doc id */

	std::vector<std::string>::const_iterator i;
	for ( i = names.begin(); i != names.end(); ++i )
	{
		Document* doc = new Document(*i);
		docs_.insert(document_map_t::value_type(*i, doc));

		/* build the document structure.
		 */
		IniFile::section_data_t data;
		ini.getSection(*i, data);

		IniFile::section_data_t::const_iterator j;
		for ( j = data.begin(); j != data.end(); ++j )
		{
			if ( (*j).first != "style" )
			{
				if ( ((*j).second == "id") || ((*j).second == "document") )
				{
					std::cerr << "ImageMaker::load() error in [" << *i <<
					             "] element definition uses a reserved word: " <<
					             (*j).second << std::endl;
					break;
				}

				Element* el = 0;

				if ( (*j).first == "text" )
				{
					el = new TextElement((*j).second);
				}
				else if ( (*j).first == "list" )
				{
					el = new ListElement((*j).second);
				}
				else if ( (*j).first == "image" )
				{
					el = new ImageElement((*j).second);
				}
				else
				{
					; /* silently ignore unknown types */
				}

				if ( el && !doc->addElement(el) )
				{
					std::cerr << "ImageMaker::load() error in [" << *i <<
				                 "] element: " <<  (*j).second <<
					             " already defined" << std::endl;
					delete el;
				}
			}
		}

		/* structure complete, apply style.
		 */
		std::string style_filename = ini.get(*i, "style", std::string());
		if ( !style_filename.empty() )
		{
			loadStyle(style_filename, *doc);
		}
	}
	return true;
}


/*
 * Load style information from a file and apply it to document.
 */
bool ImageMaker::loadStyle( const std::string& filename, Document& doc )
{
	IniFile ini;
	if ( !ini.load(filename) )
	{
		std::cerr << "ImageMaker::loadStyle() failed to open: " << filename << std::endl;
		return false;
	}

	IniFile::section_data_t data;

	/* Get the overall size of the canvas.
	 * This is required if when scaling the output.
	 */
	ini.getSection("canvas", data);
	Style canvas_style;
	readStyle(data, canvas_style);
	doc.setCanvasWidth(canvas_style.getWidth());
	doc.setCanvasHeight(canvas_style.getHeight());

	/* Process the global section first so that its style can be inherited by
 	 * the elements.
	 */
	data.clear();
	ini.getSection("document", data);

	Style doc_style;
	readStyle(data, doc_style);
	doc.setStyle(doc_style);

	doc_style.setBackgroundImage("");  /* don't inherit */

	std::vector<std::string> names;
	ini.getSectionNames(names);        /* section name => element id */

	/* Everything other than 'document' is an element id
	 */
	std::vector<std::string>::const_iterator i;
	for ( i = names.begin(); i != names.end(); ++i )
	{
		if ( (*i != "format") && (*i != "document") )
		{
			data.clear();
			ini.getSection(*i, data);
			Style style = doc_style;   /* inherit */
			readStyle(data, style);
			doc.setStyle(*i, style);
		}
	}
	return true;
}


/*
 * Read styles from the file data and save into style.
 */
void ImageMaker::readStyle( const IniFile::section_data_t data, Style& style )
{
	IniFile::section_data_t::const_iterator i;
	for ( i = data.begin(); i != data.end(); ++i )
	{
		if ( (*i).first == "height" )
		{
			style.setHeight(atoi((*i).second.c_str()));
		}
		else if ( (*i).first == "width" )
		{
			style.setWidth(atoi((*i).second.c_str()));
		}
		else if ( (*i).first == "top" )
		{
			style.setTop(atoi((*i).second.c_str()));
		}
		else if ( (*i).first == "left" )
		{
			style.setLeft(atoi((*i).second.c_str()));
		}
		else if ( (*i).first == "padding" )
		{
			style.setPadding(atoi((*i).second.c_str()));
		}
		else if ( (*i).first == "background-image" )
		{
			style.setBackgroundImage((*i).second);
		}
		else if ( (*i).first == "background-color" )
		{
			style.setBackgroundColor((*i).second);
		}
		else if ( (*i).first == "color" )
		{
			style.setColor((*i).second);
		}
		else if ( (*i).first == "border-radius" )
		{
			style.setBorderRadius(atoi((*i).second.c_str()));
		}
		else if ( (*i).first == "border-color" )
		{
			style.setBorderColor((*i).second);
		}
		else if ( (*i).first == "border-width" )
		{
			style.setBorderWidth(atoi((*i).second.c_str()));
		}
		else if ( (*i).first == "text-align" )
		{
			style.setTextAlign((*i).second);
		}
		else if ( (*i).first == "font-size" )
		{
			style.setFontSize((*i).second);
		}
		else
		{
			; /* silently ignore unknown params */
		}
	}
}


/*
 * Find a document object by its id.
 */
Document* ImageMaker::findDocument( const std::string& id ) const
{
	document_map_t::const_iterator i = docs_.find(id);
	if ( i != docs_.end() )
	{
		return (*i).second;
	}
	return 0;
}


/*
 * Find the document, add the contents and create the image file.
 */
bool ImageMaker::create( const std::string& id,
                         const ImageMaker::content_t& contents,
                         const std::string& filename,
                         unsigned int width,
                         unsigned int height )
{
	Document* doc = findDocument(id);
	if ( !doc )
	{
		std::cerr << "ImageMaker::create() document not found: \"" << id << "\"" << std::endl;
		return false;
	}

	doc->clearContent();

	content_t::const_iterator j = contents.begin();
	for ( ; j != contents.end(); ++j )
	{
		if ( doc->isElementType((*j).first, ETYPE_IMAGE) )
		{
			doc->addContent((*j).first, image_dir_ + "/" + (*j).second);
		}
		else
		{
			doc->addContent((*j).first, (*j).second);
		}
	}
	return doc->draw(filename, width, height);
}


/* vim:ts=4:set nu:
 * EOF
 */
