/**
*
* DIALOGIC CONFIDENTIAL 
* Copyright © 2012 Dialogic Inc. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to 
* the source code (collectively "Material") are owned by Dialogic Inc. and/or its subsidiaries, affiliates, suppliers
* or licensors (collectively "Dialogic"), with whom title to the Material remains. The Material contains trade secrets and proprietary
* and confidential information of Dialogic. The
* Material is protected by worldwide patent, copyright, trade secret and/or other applicable intellectual property laws and treaty
* provisions. No part of the Material may be used, copied, reproduced, 
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret and/or other intellectual 
* property right is granted to or conferred upon you by disclosure or delivery
* of the Material, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

/*!
 * \file   imagemakerthread.cxx
 * \brief  A threaded wrapper around an ImageMaker object.
 * \author John Tarlton
 */


#include "string-util.h"

#include "url.h"
#include "imagemakerthread.h"

/*
 * ctor, save the callback.
 */
ImageMakerThread::ImageMakerThread( ImageMakerThread::callback_t callback )
 : callback_ (callback)
{
	;
}


/*
 * dtor, base class will cleanup the thread.
 */
ImageMakerThread::~ImageMakerThread()
{
	;
}


/*
 * Load configuration.
 * \param conf_dir - name of the directory containing document templates.
 * \param image_dir - base directory containing the image files used for
 *                    image elements.
 */
bool ImageMakerThread::init( const std::string& conf_dir,
                             const std::string& image_dir )
{
	if ( imagemaker_.init(conf_dir, image_dir) )
	{
		start();
		return true;
	}
	return false;
}


/*
 * Parse the request string into the document id and contents.
 */
bool ImageMakerThread::create( const std::string& request,
	                           const std::string& filename,
                               unsigned int width,
                               unsigned int height,
                               void* userdata )
{
	ImageMaker::content_t contents;
	std::string id;

	std::vector<std::string> params;
	tokenise(request, params, '&');

	std::vector<std::string>::const_iterator i = params.begin();
	for ( ; i != params.end(); ++i )
	{
		std::vector<std::string> name_value;
		tokenise(*i, name_value, '=');
		if ( name_value.size() == 2 )
		{
			std::string name;
			urlDecode(name_value[0], name);

			std::string value;
			urlDecode(name_value[1], value);

			trim(name);
			if ( name == "id" )
			{
				id = value;
			}
			else
			{
				contents.push_back(ImageMaker::content_t::value_type(name, value));
			}
		}
	}
	return create(id, contents, filename, width, height, userdata);
}


/*
 * Build a request message and append it to the work queue.
 */
bool ImageMakerThread::create( const std::string& id,
                               const ImageMaker::content_t& contents,
                               const std::string& filename,
                               unsigned int width,
                               unsigned int height,
                               void* userdata )
{
	ImageMakerThreadRequest* request = new ImageMakerThreadRequest; /* deleted by run() */

	request->id = id;
	request->contents = contents;
	request->filename = filename;
	request->userdata = userdata;
	request->width = width;
	request->height = height;
	put(request);
	return true;
}


/*
 * Read a request from the work queue, process it and send a response via the
 * user's callback function.
 */
int ImageMakerThread::run()
{
	while ( !isStopping() )
	{
		ImageMakerThreadRequest* request = get(100);

		if ( request )
		{
			ImageMakerThreadResponse response;
			response.userdata = request->userdata;

			if ( imagemaker_.create(request->id,
			                        request->contents,
			                        request->filename,
			                        request->width,
			                        request->height) )
			{
				Document* doc = imagemaker_.findDocument(request->id);

				if ( request->width && request->height )
				{
					float w_scale = request->width / float(doc->getCanvasWidth());
					float h_scale = request->height / float(doc->getCanvasHeight());
					response.width = static_cast<unsigned int>(float(doc->getStyle().getWidth()) * w_scale);
					response.height = static_cast<unsigned int>(float(doc->getStyle().getHeight()) * h_scale);
					response.left = static_cast<unsigned int>(float(doc->getStyle().getLeft()) * w_scale);
					response.top = static_cast<unsigned int>(float(doc->getStyle().getTop()) * h_scale);
				}
				else
				{
					response.width = doc->getStyle().getWidth();
					response.height = doc->getStyle().getHeight();
					response.left = doc->getStyle().getLeft();
					response.top = doc->getStyle().getTop();
				}
				response.filename = request->filename;
				response.ok = true;
			}
			else
			{
				response.ok = false;
			}

			(*callback_)(&response);
			delete request;
		}
	}
	return 0;
}

/* vim:ts=4:set nu:
 * EOF
 */
