/**
*
* DIALOGIC CONFIDENTIAL 
* Copyright © 2012 Dialogic Inc. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to 
* the source code (collectively "Material") are owned by Dialogic Inc. and/or its subsidiaries, affiliates, suppliers
* or licensors (collectively "Dialogic"), with whom title to the Material remains. The Material contains trade secrets and proprietary
* and confidential information of Dialogic. The
* Material is protected by worldwide patent, copyright, trade secret and/or other applicable intellectual property laws and treaty
* provisions. No part of the Material may be used, copied, reproduced, 
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret and/or other intellectual 
* property right is granted to or conferred upon you by disclosure or delivery
* of the Material, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

/*!
 * \file   imagemakerthread.h
 * \brief  A threaded wrapper around an ImageMaker object.
 * \author John Tarlton
 */


#ifndef _IMAGEMAKERTHREAD_H
#define _IMAGEMAKERTHREAD_H

#include <string>

#include "workqueuethread.h"
#include "imagemaker.h"


/*!
 * Container for request parameters.
 * Used internally only.
 */
struct ImageMakerThreadRequest
{
	void*                  userdata;

	std::string            id;
	ImageMaker::content_t  contents;
	std::string            filename;
	unsigned int           width;
	unsigned int           height;
};


/*!
 * Container for completion data.
 */
struct ImageMakerThreadResponse
{
	void*         userdata;        /*!< userdata passed into the request */
	bool          ok;              /*!< status */

	unsigned int  top;             /*!< offset of top of image in pixels */
	unsigned int  left;            /*!< offset of left of image in pixels */
	unsigned int  height;          /*!< height of image in pixels */
	unsigned int  width;           /*!< width of image in pixels */
	std::string   filename ;       /*!< generated file, as per request */
};


/*!
 * \class ImageMakerThread
 */
class ImageMakerThread : public WorkQueueThread<ImageMakerThreadRequest>
{
public:

	/*!
	 * This defines the signature of the callback function that will receive
	 * response events when a request has been completed.
	 * The contents of the response structure are valid until the callback
	 * function returns; the application must make copies of any parts of the
	 * response structure that it requires access to after this time.
	 */
	typedef void (*callback_t)(const ImageMakerThreadResponse*);

	/*!
	 * ctor
	 * \param callback - application's callback for receiving completion events.
	 */
	ImageMakerThread( ImageMakerThread::callback_t callback );

	/*!
	 * dtor
	 */
	virtual ~ImageMakerThread();

	/*!
	 * Load configuration and start the worker thread.
	 * \param conf_dir - name of the directory containing document templates.
	 * \param image_dir - base directory containing the image files used for
	 *                    image elements.
	 */
	bool init( const std::string& conf_dir, const std::string& image_dir );

	/*!
	 * Create an image and store in the named file.
	 * \param request - a list of name/value pairs seperated by ampersands
	 *                  in the same format as a http GET uri string.
	 *                  E.g. "id=mydoc&element=value&element=value&..."
	 *                  The name 'id' is mandatory and is used to identify the
	 *                  base document.
	 * \param filename - name of the resulting image file.
	 * \param width - if non-zero this overrides the width of the image.
	 * \param height - if non-zero this overrides the height of the image.
	 * \param userdata - application defined value returned in the response.
	 */
	bool create( const std::string& request,
	             const std::string& filename,
	             unsigned int width = 0,
	             unsigned int height = 0,
	             void* userdata = (void*)(0) );

	/*!
	 * Create an image using the template 'id' and data 'contents' and store
	 * it in the named file.
	 * \param id  - document template .
	 * \param contents - content information.
	 * \param filename - name of the resulting image file.
	 * \param width - if non-zero this overrides the width of the image.
	 * \param height - if non-zero this overrides the height of the image.
	 * \param userdata - application defined value returned in the response.
	 */
	bool create( const std::string& id,
	             const ImageMaker::content_t& contents,
 	             const std::string& filename,
	             unsigned int width = 0,
	             unsigned int height = 0,
	             void* userdata = (void*)(0) );

private:

	ImageMakerThread();    /* application must supply a callback */

	virtual int run();

private:

	ImageMaker imagemaker_;
	callback_t callback_;
};


#endif // _IMAGEMAKERTHREAD_H

/* vim:ts=4:set nu:
 * EOF
 */
