/**
*
* DIALOGIC CONFIDENTIAL 
* Copyright © 2012 Dialogic Inc. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to 
* the source code (collectively "Material") are owned by Dialogic Inc. and/or its subsidiaries, affiliates, suppliers
* or licensors (collectively "Dialogic"), with whom title to the Material remains. The Material contains trade secrets and proprietary
* and confidential information of Dialogic. The
* Material is protected by worldwide patent, copyright, trade secret and/or other applicable intellectual property laws and treaty
* provisions. No part of the Material may be used, copied, reproduced, 
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret and/or other intellectual 
* property right is granted to or conferred upon you by disclosure or delivery
* of the Material, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

/*!
 * \file   string-util.h
 * \brief  Utilities for manipulating strings.
 * \author John Tarlton
 */

#ifndef _STRING_UTIL_H
#define _STRING_UTIL_H

/*------------------------------- Dependencies -------------------------------*/

#include <vector>
#include <map>
#include <string>
#include <string.h>

/*----------------------------------------------------------------------------*/

/*!
 * Copy an stl string to a null terminated character array.
 *
 * \param src - source string.
 * \param dst - array where the sequence of characters is copied.
 * \param limit - maximum number of characters to be copied, including the
 *                null character.
 * \return a pointer to the array.
 */
inline char* copy_string_to_c_array( const std::string& src,
                                     char* dst,
                                     size_t limit )
{
	if ( src.length() < limit - 1 )
	{
		strcpy(dst, src.c_str());
	}
	else
	{
		strcpy(dst, src.substr(0, limit - 1).c_str());
		dst[limit - 1] = '\0';
	}
	return dst;
}


/*!
 * Parse a string into a sequence of tokens
 *
 * \param src - source string.
 * \param tokens - parsed tokens.
 * \param delim - the character that delimits the tokens in the src string.
 */
void tokenise( const std::string& src,
               std::vector<std::string>& tokens,
               char delim );


/*!
 * Remove leading and trailing spaces from a string.
 * \param s - string to be trimmed.
 */
void trim( std::string& s );


/*!
 * \class StringTokeniser
 * Simple string tokeniser.
 */
class StringTokeniser
{
public:

	StringTokeniser( const std::string& s )
	 : s_ (s), pos_ (0)
	{
		;
	}

	/*!
	 * Get the next token from the source string.
	 * \param delim  - delimiting character
	 * \return a new string containing the next token. This string does not
	 *         include the delimiting character.
	 */
	std::string token( char delim )
	{
		std::string::size_type endpos = s_.find(delim, pos_);
		if ( endpos != std::string::npos )
		{
			std::string::size_type startpos = pos_;
			pos_ = endpos + 1;
			return s_.substr(startpos, endpos - startpos);
		}
		return std::string();
	}

	/*!
	 * Get the next token from the source string.
	 * \param delims  - set of delimiting characters.
	 * \return a new string containing the next token. This string does not
	 *         include the delimiting character.
	 */
	std::string token( const char* delims )
	{
		std::string::size_type endpos = s_.find_first_of(delims, pos_);
		if ( endpos != std::string::npos )
		{
			std::string::size_type startpos = pos_;
			pos_ = endpos + 1;
			return s_.substr(startpos, endpos - startpos);
		}
		return std::string();
	}

	/*!
	 * Get the the remainder of the string.
	 * \return a new string containing the remainder of the source string.
	 */
	std::string token()
	{
		return s_.substr(pos_);
	}

private:

	std::string            s_;
	std::string::size_type pos_;
};


#endif //  _STRING_UTIL_H

/* vim:ts=4:set nu:
 * EOF
 */
