/**
*
* DIALOGIC CONFIDENTIAL 
* Copyright © 2012 Dialogic Inc. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to 
* the source code (collectively "Material") are owned by Dialogic Inc. and/or its subsidiaries, affiliates, suppliers
* or licensors (collectively "Dialogic"), with whom title to the Material remains. The Material contains trade secrets and proprietary
* and confidential information of Dialogic. The
* Material is protected by worldwide patent, copyright, trade secret and/or other applicable intellectual property laws and treaty
* provisions. No part of the Material may be used, copied, reproduced, 
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret and/or other intellectual 
* property right is granted to or conferred upon you by disclosure or delivery
* of the Material, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

/*!
 * \file        workqueuethread.h
 * \brief       Template class for a thread with a work queue.
 * \author      John Tarlton <john.tarlton@dialogic.com>
 * \version     14 June 2010
 */


#ifndef _WORKQUEUETHREAD_H
#define _WORKQUEUETHREAD_H

#include <pthread.h>
#include <semaphore.h>
#include <sys/time.h>

#include <queue>


/*!
 * \class WorkQueueThread
 * T defines the type of the work queue data.
 */
template<typename T>
class WorkQueueThread
{
public:
	
	/*!
	 * dtor
	 */
	virtual ~WorkQueueThread()
	{
		stop();

		pthread_mutex_unlock(&guard_);
		pthread_mutex_destroy(&guard_);
		sem_destroy(&stop_);
		sem_destroy(&ready_);
	}

	/*!
	 * Create and start a thread.
	 */
	bool start()
	{
		if ( is_running_ )
		{
			return false;
		}
		exit_code_ = 0;

		if ( 0 == pthread_create(&thread_, 0, begin, (void*)this) )
		{
			is_running_ = true;
		}
		return is_running_;
	}

	/*!
	 * Request the thread to stop and wait to it to exit.
	 */
	void stop()
	{
		if ( is_running_ )
		{
			sem_post(&stop_);
		
			if ( pthread_join(thread_, NULL) == 0 )
			{
				is_running_ = false;
			}
		}
	}

	/*!
	 * Accessor for retrieving the value returned from run().
	 */
	int getExitCode() const 
	{ 
		return exit_code_; 
	}

	/*!
	 * Append an item to the work queue.
	 */
	void put( T* data )
	{
		pthread_mutex_lock(&guard_);
		queue_.push(data);
		sem_post(&ready_);
		pthread_mutex_unlock(&guard_);
	}

protected:

	/*!
	 * ctor. The thread is created by start().
	 */
	WorkQueueThread() 
	 : is_running_ (false)
	{
		sem_init(&ready_, 0, 0);
		sem_init(&stop_, 0, 0);
		pthread_mutex_init(&guard_, NULL);
	}

	WorkQueueThread( const WorkQueueThread& );
	WorkQueueThread& operator = ( const WorkQueueThread& );

	/*!
	 * Thread body.
	 */
	virtual int run() = 0;

	/*!
	 * Must be checked by run() to test if it should exit.
	 */
	bool isStopping()
	{
		return ( sem_trywait(&stop_) == 0 );
	}

	/*!
	 * Retreive the item at the head of the queue.
	 * \param ms - time in millseconds to wait for an item to become available.
	 * \return - the item from the head of the queue or 0 if the timeout has
	 *           expired.
	 */
	T* get( unsigned long timeout )
	{
		T* data = 0;

		struct timeval now;
		gettimeofday(&now, 0);

		struct timespec abstime;
		abstime.tv_sec = now.tv_sec;
		abstime.tv_nsec = now.tv_usec * 1000;

		abstime.tv_sec += timeout / 1000;
		abstime.tv_nsec += timeout % 1000 * 1000000L;
		if ( abstime.tv_nsec >= 1000000000L )
		{
			abstime.tv_nsec -= 1000000000L;
			abstime.tv_sec++;
		}
		if ( sem_timedwait(&ready_, &abstime) == 0 )
		{
			pthread_mutex_lock(&guard_);
			if ( !queue_.empty() )
			{
				data = queue_.front();
				queue_.pop();
			}
			pthread_mutex_unlock(&guard_);
		}
		return data;
	}

private: 

	/*!
	 * Static entry point for the thread, this is passed into pthread_create().
	 */
	static void* begin( void* arg )
	{
		WorkQueueThread* thread = static_cast<WorkQueueThread*>(arg) ;
		thread->exit_code_ = thread->run();
		return (void*)(0);
	}

private:

	pthread_t        thread_;
	bool             is_running_;
	sem_t            stop_;
	int              exit_code_;

	sem_t            ready_;
	pthread_mutex_t  guard_;
	std::queue<T*>   queue_;
};


#endif  // _WORKQUEUETHREAD_H

/* EOF */

